@@ -19,14 +19,14 @@
 /// b.start_node(Syntax::Root);
 ///
 /// b.start_node(Syntax::Number);
-/// b.token(Syntax::Lit, Span::new(1, 2));
+/// b.token(Syntax::Lit, 1);
 /// b.end_node()?;
 ///
-/// b.token(Syntax::Whitespace, Span::new(2, 5));
+/// b.token(Syntax::Whitespace, 3);
 ///
 /// b.start_node(Syntax::Number);
-/// b.token(Syntax::Lit, Span::new(5, 7));
-/// b.token(Syntax::Lit, Span::new(7, 9));
+/// b.token(Syntax::Lit, 2);
+/// b.token(Syntax::Lit, 2);
 /// b.end_node()?;
 ///
 /// b.end_node()?;
@@ -34,16 +34,16 @@
 /// let tree = b.build()?;
 ///
 /// let expected = syntree::tree! {
-///     >> Syntax::Root,
-///         >> Syntax::Number,
-///             + (1, 2) Syntax::Lit,
-///         <<
-///         + (2, 5) Syntax::Whitespace,
-///         >> Syntax::Number,
-///             + (5, 7) Syntax::Lit,
-///             + (7, 9) Syntax::Lit,
-///         <<
-///     <<
+///     Syntax::Root => {
+///         Syntax::Number => {
+///             (Syntax::Lit, 1)
+///         },
+///         (Syntax::Whitespace, 3),
+///         Syntax::Number => {
+///             (Syntax::Lit, 2),
+///             (Syntax::Lit, 2)
+///         }
+///     }
 /// };
 ///
 /// assert_eq!(expected, tree);
@@ -51,29 +51,41 @@
 /// ```
 #[macro_export]
 macro_rules! tree {
-    (@o $b:ident, == $expr:expr, $($tt:tt)*) => {{
+    (@o $b:ident,) => {};
+
+    (@o $b:ident, ($expr:expr, $len:expr) $(,)?) => {{
+        $b.token($expr, $len);
+    }};
+
+    (@o $b:ident, ($expr:expr, $len:expr), $($rest:tt)*) => {{
+        $b.token($expr, $len);
+        $crate::tree!(@o $b, $($rest)*);
+    }};
+
+    (@o $b:ident, $expr:expr $(,)?) => {{
         $b.start_node($expr);
         $b.end_node()?;
-        $crate::tree!(@o $b, $($tt)*);
     }};
 
-    (@o $b:ident, >> $expr:expr, $($tt:tt)*) => {{
+    (@o $b:ident, $expr:expr, $($rest:tt)*) => {{
+        $b.start_node($expr);
+        $b.end_node()?;
+        $crate::tree!(@o $b, $($rest)*);
+    }};
+
+    (@o $b:ident, $expr:expr => { $($tt:tt)* } $(,)?) => {{
         $b.start_node($expr);
         $crate::tree!(@o $b, $($tt)*);
-    }};
-
-    (@o $b:ident, << $(,)? $($tt:tt)*) => {{
         $b.end_node()?;
-        $crate::tree!(@o $b, $($tt)*);
     }};
 
-    (@o $b:ident, + ($start:expr, $end:expr) $expr:expr, $($tt:tt)*) => {{
-        $b.token($expr, $crate::Span::new($start, $end));
+    (@o $b:ident, $expr:expr => { $($tt:tt)* }, $($rest:tt)*) => {{
+        $b.start_node($expr);
         $crate::tree!(@o $b, $($tt)*);
+        $b.end_node()?;
+        $crate::tree!(@o $b, $($rest)*);
     }};
 
-    (@o $b:ident, $(,)?) => {};
-
     ($($tt:tt)*) => {{
         let mut b = $crate::TreeBuilder::new();
         $crate::tree!(@o b, $($tt)*);
